from chronometry.date import get_today
import re


RESOLUTION_MARKERS = [
	'2160p', '1440p', '1080p', '720p', '480p', '360p', '240p',
	'4k', '8k'
]


def is_resolution(string):
	return string.lower() in RESOLUTION_MARKERS or f'[{string.lower()}]' in RESOLUTION_MARKERS


def is_year(string, year_min, year_max):
	string = str(string)

	if re.search(r'^\(\d{4}\)$', string):
		return year_min <= int(string.strip('()')) <= year_max
	elif re.search(r'^\d{4}$', string):
		return year_min <= int(string) <= year_max
	else:
		return False


def get_title_year_and_resolution(name, sep=None, year_min=1950):
	year_max = get_today().year

	title_and_year_part = []
	resolution_part = []

	title_part = True

	if sep is None:
		if len(name.split(' ')) > len(name.split('.')):
			sep = ' '
		else:
			sep = '.'

	for x in name.split(sep):
		if is_resolution(x):
			title_part = False

		if title_part:
			title_and_year_part.append(x)

		elif is_resolution(x):
			resolution_part.append(x.lower().strip('[]'))

	title_part_reversed = []
	year = None

	for x in reversed(title_and_year_part):
		if year is None:
			if is_year(string=x, year_min=year_min, year_max=year_max):
				year = int(x.strip('()'))

		else:
			title_part_reversed.append(x)

	title_part = reversed(title_part_reversed)

	return {
		'title': ' '.join(title_part),
		'year': year,
		'resolution': ' '.join(resolution_part)
	}














from disk import Path
from .is_movie_file import is_movie_file
from .MovieFile import get_all_files
from .MovieFile import MovieFile
from .MovieFile import MovieDirectory
from martin.IMDB import IMDB


class CollectionDirectory(Path):
	def __init__(self, path=None, string=None, imdb=None):
		path = path or string
		super().__init__(string=path, show_size=False)

		self._movie_files = None
		self._imdb = imdb or IMDB()

	@property
	def movie_files(self):
		"""
		:rtype: list[MovieFile]
		"""
		if self._movie_files is None:
			self._movie_files = [
				MovieFile(file, imdb=self.imdb) for file in get_all_files(self)
				if is_movie_file(file)
			]

		return self._movie_files

	@property
	def movie_directories(self):
		"""
		:rtype: list[MovieDirectory]
		"""
		return [movie_file.directory for movie_file in self.movie_files]

	def reset(self):
		self._movie_files = None

	def reset_original_names_files(self):
		for movie_file in self.movie_files:
			movie_file.reset_original_names()

	@property
	def imdb(self):
		"""
		:rtype: IMDB
		"""
		return self._imdb



from disk import HardFolder
from disk import Path

from .IMDB import IMDB

_IMDB = IMDB()


class MovieCollection:
	def __init__(self, path, imdb=None):
		"""
		:type path: str or Path
		"""

		# path
		self._path = Path(path)
		if not self.path.exists():
			self.path.make_directory()
		elif self.path.is_file():
			raise TypeError(f'{path.path} is a file!')
		self._imdb = imdb or _IMDB

		# information
		self._data_folder = HardFolder(path=self.path + 'data')
		self.data['directories'] = []
		self.data['movie_files'] = []
		self.data['genres'] = []
		self.data['logs'] = []

	def __getstate__(self):
		return self._path

	def __setstate__(self, state):
		self._path = state
		self._imdb = IMDB()
		self._data_folder = HardFolder(path=self.path + 'data')

	@property
	def path(self):
		"""
		:rtype: Path
		"""
		return self._path

	@property
	def data(self):
		"""
		:rtype: HardFolder
		"""
		return self._data_folder

	@property
	def directories(self):
		"""
		:rtype: list[Path]
		"""
		return self.data['directories']

	def add_directory(self, path):
		path = Path(path)
		if not path.exists():
			raise NotADirectoryError(f'{path.path} does not exist!')
		elif path.is_file():
			raise NotADirectoryError(f'{path.path} is a file!')

		directories = self.directories
		if not path in directories:
			directories.append(path)
		self.data['directories'] = directories













from disk import Path
from imdb.Movie import Movie

from .is_movie_file import is_movie_file
from .is_movie_file import is_movie_directory
from .get_title_year_and_resolution import get_title_year_and_resolution
from martin.IMDB import IMDB


def get_all_files(path):
	"""
	:type path: Path
	:rtype: list[Path]
	"""
	if path.is_file():
		return [path]

	else:
		return [f for p in path.list() for f in get_all_files(p)]


class MovieDirectory(Path):
	def __init__(self, path=None, string=None, movie_file=None):
		path = path or string
		super().__init__(string=path)
		if not is_movie_directory(self):
			raise TypeError(f'{self.path} is not a movie directory!')

		if movie_file is None:
			movie_file = [file for file in self.files if is_movie_file(file)][0]
		self._movie_file = movie_file

	@property
	def movie_file(self):
		"""
		:rtype: MovieFile
		"""
		return self._movie_file

	@property
	def contents(self):
		"""
		:rtype: list[Path]
		"""
		return [self.movie_file] + [
			x for x in self.list() if x.path != self.movie_file.path
		]

	@property
	def title(self):
		return self.movie_file.title

	@property
	def year(self):
		return self.movie_file.year

	@property
	def genres(self):
		return self.movie_file.genres


class MovieFile(Path):
	def __init__(self, path=None, string=None, imdb=None):
		path = path or string
		super().__init__(string=path)
		if not is_movie_file(self):
			raise TypeError(f'{self.path} is not a movie file!')

		if is_movie_directory(self.parent_directory):
			self._directory = MovieDirectory(
				path=self.parent_directory, movie_file=self
			)

		else:
			self._directory = None

		self._directory_name_original = None
		self.fix_directory()
		self._original_names_file = None
		self._original_names = None
		self._imdb = imdb or IMDB()
		self._movie = None
		self._title = None
		self._year = None

	def __repr__(self):
		if self.directory is None:
			return self.description
		else:
			return f'{self.directory.__repr__()}\n\t{self.description}'

	@property
	def directory(self):
		"""
		:rtype: MovieDirectory or NoneType
		"""
		return self._directory

	@property
	def accompanying_files_and_directories(self):
		"""
		:rtype: list[Path]
		"""
		if self.directory is None:
			return []

		else:
			return [x for x in self.directory.list() if x.path != self.path]

	def fix_directory(self):
		if self.directory is None:
			self._directory_name_original = False
			print(f'fixing {self.path}')
			self.move(new_directory=self.parent_directory + self.name.replace('.', ' '))
			self._directory = MovieDirectory(
				path=self.parent_directory, movie_file=self
			)
		else:
			self._directory_name_original = True

	def reset_original_names(self):
		self._original_names = None
		self.original_names_file.delete()

	def create_original_names_file(self):
		self.reset_original_names()
		self.original_names_file.save(self.original_names)

	@property
	def original_names_file(self):
		"""
		:rtype: Path
		"""
		if self._original_names_file is None:
			self._original_names_file = self.directory + 'original_names.pickle'
		return self._original_names_file

	@property
	def original_names(self):
		"""
		:rtype: dict[str, str]
		"""
		if self._original_names is None:
			if self.original_names_file.exists():
				self._original_names = self.original_names_file.load()

			elif self._directory_name_original:
				self._original_names = {
					'file': self.name_and_extension,
					'directory': self.directory.name_and_extension
				}

			else:
				self._original_names = {
					'file': self.name_and_extension,
					'directory': None
				}

		return self._original_names

	@property
	def original_title_year_resolution(self):
		"""
		:rtype: dict[str, str or int]
		"""
		original_names = self.original_names

		if original_names['directory'] is not None:
			name_year_resolution = {
				**get_title_year_and_resolution(name=original_names['file']),
				**get_title_year_and_resolution(name=original_names['directory'])
			}

		else:
			name_year_resolution = get_title_year_and_resolution(name=original_names['file'])

		return name_year_resolution

	@property
	def original_title(self):
		return self.original_title_year_resolution['title']

	@property
	def original_year(self):
		return self.original_title_year_resolution['year']

	@property
	def original_resolution(self):
		return self.original_title_year_resolution['resolution']

	@property
	def description(self):
		return f'{self.title} ({self.year}) [{", ".join(self.genres)}]'

	@property
	def imdb(self):
		"""
		:rtype: IMDB
		"""
		return self._imdb

	@property
	def movie(self):
		"""
		:rtype: Movie or False
		"""
		if self._movie is None:
			self._movie = self.imdb.search(
				title=self.original_title, year=self.original_year
			)
		return self._movie

	@property
	def movie_data_file(self):
		if self.directory is None:
			raise RuntimeError('directory does not exist!')
		movie_data_file = self.directory + 'movie_data.pickle'
		if not movie_data_file.exists():
			if self.movie:
				movie_data_file.save({
					'title': self.movie['title'],
					'year': self.movie['year'],
					'genres': self.movie.get('genres')
				})
			else:
				movie_data_file.save(False)
		return movie_data_file

	@property
	def movie_data(self):
		return self.movie_data_file.load()

	@property
	def title(self):
		"""
		:rtype: str
		"""
		if self.movie_data:
			return self.movie_data['title']

		else:
			return self.original_title

	@property
	def year(self):
		"""
		:rtype: int
		"""
		if self.movie_data:
			return self.movie_data['year']

		else:
			return self.original_year

	@property
	def genres(self):
		"""
		:rtype: list[str]
		"""
		if self.movie_data:
			genres = self.movie_data['genres']
			if genres is None:
				return []

			else:
				return list(genres)

		else:
			return []



from .IMDB import IMDB

_IMDB = IMDB()


class Movie:
	def __init__(self, title=None, year=None, imdb=None):
		"""
		:type movie: _Movie
		"""
		self._imdb = imdb or _IMDB
		self._title = None
		self._year = None
		self._genres = None

		self._movie = self._imdb.search(title=title, year=year)

	def __eq__(self, other):
		if not isinstance(other, Movie):
			raise TypeError(f'cannot compare a Movie with a {type(other)}')
		return self.year == other.year and self.title == other.title

	def __ge__(self, other):
		if not isinstance(other, Movie):
			raise TypeError(f'cannot compare a Movie with a {type(other)}')
		return (self.year, self.title) >= (other.year, other.title)

	def __le__(self, other):
		if not isinstance(other, Movie):
			raise TypeError(f'cannot compare a Movie with a {type(other)}')
		return (self.year, self.title) <= (other.year, other.title)

	def __ne__(self, other):
		return not self == other

	def __gt__(self, other):
		return not self <= other

	def __lt__(self, other):
		return not self >= other

	@property
	def title(self):
		"""
		:rtype: str
		"""
		if self._title is None:
			self._title = str(self._movie['title'])
		return self._title

	@property
	def year(self):
		"""
		:rtype: int
		"""
		if self._year is None:
			self._year = int(self._movie['year'])
		return self._year

	@property
	def genres(self):
		"""
		:rtype: list[str]
		"""
		if self._genres is None:
			genres = self._movie.get('genres')
			if genres is None:
				self._genres = []
			else:
				self._genres = list(genres)
		return self._genres

	def genre_is(self, genre):
		"""
		:type genre: str
		:rtype: bool
		"""
		return genre.lower() in [x.lower() for x in self.genres]

	def genre_in(self, genres):
		"""
		:type genres: list[str]
		:rtype: bool
		"""
		return any([self.genre_is(genre=genre) for genre in genres])




from disk import Path
MOVIE_FILE_EXTENSIONS = [
	'avi', 'mp4', 'mkv', 'wmv', 'mov', 'flv', 'm4p', 'm4v', 'ogg', 'webm',
	'mpeg', 'mpg', 'mp2', 'mpe', 'mpv', 'qt', 'swf', 'avchd'
]


def is_movie_file(path):
	"""
	:type path: Path
	:rtype: bool
	"""

	if path.is_directory():
		return False

	elif not path.exists():
		return False

	elif path.extension.lower() in MOVIE_FILE_EXTENSIONS and path.get_size_gb() >= 1:
		return True

	else:
		return False


def has_movie_file(directory):
	"""
	:type directory: Path
	:rtype: bool
	"""

	for path in directory.files:
		if is_movie_file(path):
			return True

	for path in directory.directories:
		if has_movie_file(path):
			return True

	return False


def get_num_movie_files(path):
	"""
	:type path: Path
	:rtype: bool
	"""
	if path.is_file():
		if is_movie_file(path):
			return 1
		else:
			return 0

	else:
		return sum([get_num_movie_files(subpath) for subpath in path.list()])


def is_movie_directory(path):
	"""
	:type path: Path
	:rtype: bool
	"""
	if path.is_file():
		return False

	elif not path.exists():
		return False

	else:
		for subpath in path.directories:
			if has_movie_file(subpath):
				return False

		return sum([1 for file in path.files if is_movie_file(file)]) == 1




from imdb import IMDb
from imdb.Movie import Movie


class IMDB:
	def __init__(self):
		self._imdb = IMDb()

	def search(self, title, year=None):
		"""
		:type title: str
		:type year: int
		:rtype: Movie
		"""
		movies = self._imdb.search_movie(title=title)

		if len(movies) == 0:
			print(f'{title} ({year}) not found!')
			return False

		if year is not None:
			for movie in movies:
				if movie.data['year'] == year:
					return self._imdb.get_movie(movie.movieID)

		return self._imdb.get_movie(movies[0].movieID)
